import { autoinject, Container }   from "aurelia-dependency-injection";
import { PLATFORM, viewResources } from "aurelia-framework";
import { bindable }                from "aurelia-typed-observable-plugin";

const VIEW_ACTIVE_LEAVE_CLASS: string = "au-leave-active";

@autoinject
@viewResources( PLATFORM.moduleName( "./pitaya-view.css" ) )
export class PitayaView
{
	@bindable public noScrolling: boolean;
	@bindable public noInteraction: boolean;
	@bindable public onAnimated: Function;
	@bindable public onAttached: Function;

	private _animationReady: boolean = false;

	private _innerElement: HTMLElement;

	get innerElement(): HTMLElement
	{
		return this._innerElement;
	}

	get container(): Container
	{
		return this._container;
	}

	constructor(
		private _container: Container,
		private _rootElement: Element
	)
	{}

	public scrollTo( target: number | HTMLElement, offset: number = 16 )
	{
		const viewElement = this._rootElement;
		const position    = target instanceof HTMLElement ? Math.max( 0, target.offsetTop - offset ) : target;

		/* Does not work in every browser (safari)
		* See: https://caniuse.com/#feat=css-scroll-behavior*/
		viewElement.scrollTo( {
			behavior : "smooth",
			left     : 0,
			top      : position
		} );
	}

	public async reinitialize(): Promise<void>
	{
		await this.attached();
	}

	private attached()
	{
		this._setAnimationListener();

		this.onAttached && this.onAttached( { component : this } );
	}

	private _willBeAnimated(): boolean
	{
		// @ts-ignore
		const siblings = [ ...this._rootElement.parentElement.children ];

		return siblings.some( sibling => sibling !== this._rootElement && sibling.classList.contains( VIEW_ACTIVE_LEAVE_CLASS ) );
	}

	private _setAnimationListener()
	{
		if( !this._willBeAnimated() )
		{
			this._onAnimationEnd();

			return;
		}

		this._rootElement.addEventListener( "animationend", this._onAnimationEnd.bind( this ), { once : true } );
	}

	private _onAnimationEnd()
	{
		this._animationReady = true;

		this.onAnimated && this.onAnimated( { component : this } );
	}
}