import { HttpClient } from "aurelia-fetch-client";
import { PLATFORM }   from "aurelia-framework";

const httpClient           = new HttpClient();
const _GUID_PREFIX: string = "pty-";
let _idCounter: number     = 0;

export const delay = function( ms: number )
{
	return new Promise( ( resolve ) => window.setTimeout( resolve, ms ) );
};

export const nextAnimationFrame = function()
{
	return new Promise( ( resolve ) => requestAnimationFrame( resolve ) );
};

export const nextEventRaise = function( element: HTMLElement, eventName: string ): Promise<void>
{
	return new Promise( ( resolve ) =>
	{
		const callback = ( { srcElement }: Event ) =>
		{
			if( srcElement !== element )
			{
				return;
			}

			element.removeEventListener( eventName, callback );

			resolve();
		};

		element.addEventListener( eventName, callback );
	} );
};

export const createElement = function( htmlString: string ): Node
{
	const template: HTMLTemplateElement = document.createElement( "template" );

	template.innerHTML = htmlString.trim();

	return template.content.firstChild;
};

export const getJSON = async ( url: string ) =>
{
	const response = await httpClient.fetch( PLATFORM.moduleName( url ) );
	const status   = response.status;

	if( status >= 200 && status < 300 )
	{
		return response.json();
	}
	else
	{
		// throw the object you desire which you will handle
		// in this case the json error response might give us
		// a json body so we can use that.
		throw {
			status,
			body : await response.json()
		};
		// or throw an error
		// throw Error(await response.statusText)
	}
};

export const generateGUID = function(): string
{
	return `${_GUID_PREFIX}${_idCounter++}`;
};